@@ -1,7 +1,7 @@
 /*
  *  ClamAV bytecode handler tool.
  *
- *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -52,7 +52,7 @@ static void help(void)
     printf("\n");
     printf("                       Clam AntiVirus: Bytecode Testing Tool %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2021 Cisco Systems, Inc.\n");
+    printf("           (C) 2022 Cisco Systems, Inc.\n");
     printf("\n");
     printf("    clambc <file> [function] [param1 ...]\n");
     printf("\n");
@@ -367,6 +367,7 @@ int main(int argc, char *argv[])
         struct cl_engine *engine = cl_engine_new();
         fmap_t *map              = NULL;
         memset(&cctx, 0, sizeof(cctx));
+
         if (!engine) {
             fprintf(stderr, "Unable to create engine\n");
             optfree(opts);
@@ -394,11 +395,21 @@ int main(int argc, char *argv[])
         }
         ctx->ctx    = &cctx;
         cctx.engine = engine;
-        cctx.fmap   = cli_calloc(sizeof(fmap_t *), engine->maxreclevel + 2);
-        if (!cctx.fmap) {
+
+        cctx.recursion_stack_size = cctx.engine->max_recursion_level;
+        cctx.recursion_stack      = cli_calloc(sizeof(recursion_level_t), cctx.recursion_stack_size);
+        if (!cctx.recursion_stack) {
             fprintf(stderr, "Out of memory\n");
             exit(3);
         }
+
+        // ctx was memset, so recursion_level starts at 0.
+        cctx.recursion_stack[cctx.recursion_level].fmap = map;
+        cctx.recursion_stack[cctx.recursion_level].type = CL_TYPE_ANY; /* ANY for the top level, because we don't yet know the type. */
+        cctx.recursion_stack[cctx.recursion_level].size = map->len;
+
+        cctx.fmap = cctx.recursion_stack[cctx.recursion_level].fmap;
+
         memset(&dbg_state, 0, sizeof(dbg_state));
         dbg_state.file     = "<libclamav>";
         dbg_state.line     = 0;
@@ -466,7 +477,7 @@ int main(int argc, char *argv[])
         if (map)
             funmap(map);
         cl_engine_free(engine);
-        free(cctx.fmap);
+        free(cctx.recursion_stack);
     }
     cli_bytecode_destroy(bc);
     cli_bytecode_done(&bcs);
