@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -572,11 +572,12 @@ static int cli_cvdverify(FILE *fs, struct cl_cvd *cvdpt, unsigned int skipsig)
     return CL_SUCCESS;
 }
 
-int cl_cvdverify(const char *file)
+cl_error_t cl_cvdverify(const char *file)
 {
     struct cl_engine *engine;
     FILE *fs;
-    int ret, dbtype = 0;
+    cl_error_t ret;
+    int dbtype = 0;
 
     if ((fs = fopen(file, "rb")) == NULL) {
         cli_errmsg("cl_cvdverify: Can't open file %s\n", file);
@@ -669,7 +670,7 @@ int cli_cvdload(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigne
     if (cvd.fl > cl_retflevel()) {
         cli_warnmsg("*******************************************************************\n");
         cli_warnmsg("***  This version of the ClamAV engine is outdated.             ***\n");
-        cli_warnmsg("***   Read https://www.clamav.net/documents/installing-clamav   ***\n");
+        cli_warnmsg("***   Read https://docs.clamav.net/manual/Installing.html       ***\n");
         cli_warnmsg("*******************************************************************\n");
     }
 
