@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -33,10 +33,10 @@
 
 #define CLI_DEFAULT_BM_OFFMODE_FSIZE 262144
 
-#define CLI_DEFAULT_TIMELIMIT     120000
-#define CLI_DEFAULT_MAXSCANSIZE   104857600
-#define CLI_DEFAULT_MAXFILESIZE   26214400
-#define CLI_DEFAULT_MAXRECLEVEL   16
+#define CLI_DEFAULT_TIMELIMIT     (1000 * 60 * 2)       // 2 minutes
+#define CLI_DEFAULT_MAXSCANSIZE   (1024 * 1024 * 100)   // 100 MB
+#define CLI_DEFAULT_MAXFILESIZE   (1024 * 1024 * 25)    // 25 MB
+#define CLI_DEFAULT_MAXRECLEVEL   17
 #define CLI_DEFAULT_MAXFILES      10000
 #define CLI_DEFAULT_MIN_CC_COUNT  3
 #define CLI_DEFAULT_MIN_SSN_COUNT 3
