@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Kevin Lin <kevlin2@cisco.com>
  *
@@ -35,9 +35,11 @@ struct tiff_ifd {
     uint32_t value;
 };
 
-int cli_parsetiff(cli_ctx *ctx)
+cl_error_t cli_parsetiff(cli_ctx *ctx)
 {
-    fmap_t *map = *ctx->fmap;
+    cl_error_t status = CL_ERROR;
+
+    fmap_t *map = NULL;
     unsigned char magic[4];
     int big_endian;
     uint32_t offset = 0, ifd_count = 0;
@@ -47,37 +49,58 @@ int cli_parsetiff(cli_ctx *ctx)
 
     cli_dbgmsg("in cli_parsetiff()\n");
 
+    if (NULL == ctx) {
+        cli_dbgmsg("TIFF: passed context was NULL\n");
+        status = CL_EARG;
+        goto done;
+    }
+    map = ctx->fmap;
+
     /* check the magic */
-    if (fmap_readn(map, magic, offset, 4) != 4)
-        return CL_SUCCESS;
+    if (fmap_readn(map, magic, offset, 4) != 4) {
+        status = CL_CLEAN;
+        goto done;
+    }
     offset += 4;
 
     if (!memcmp(magic, "\x4d\x4d\x00\x2a", 4))
         big_endian = 1;
     else if (!memcmp(magic, "\x49\x49\x2a\x00", 4))
         big_endian = 0;
-    else
-        return CL_SUCCESS; /* Not a TIFF file */
+    else {
+        status = CL_CLEAN; /* Not a TIFF file */
+        goto done;
+    }
 
     cli_dbgmsg("cli_parsetiff: %s-endian tiff file\n", big_endian ? "big" : "little");
 
     /* acquire offset of first IFD */
-    if (fmap_readn(map, &offset, offset, 4) != 4)
-        return CL_EPARSE;
+    if (fmap_readn(map, &offset, offset, 4) != 4) {
+        cli_dbgmsg("cli_parsetiff: Failed to acquire offset of first IFD, file appears to be truncated.\n");
+        cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingFirstIFDOffset");
+        status = CL_EPARSE;
+        goto done;
+    }
     offset = tiff32_to_host(big_endian, offset);
 
     cli_dbgmsg("cli_parsetiff: first IFD located @ offset %u\n", offset);
 
     if (!offset) {
-        cli_errmsg("cli_parsetiff: invalid offset for first IFD\n");
-        return CL_EPARSE;
+        cli_errmsg("cli_parsetiff: Invalid offset for first IFD\n");
+        cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.InvalidIFDOffset");
+        status = CL_EPARSE;
+        goto done;
     }
 
     /* each IFD represents a subfile, though only the first one normally matters */
     do {
         /* acquire number of directory entries in current IFD */
-        if (fmap_readn(map, &num_entries, offset, 2) != 2)
-            return CL_EPARSE;
+        if (fmap_readn(map, &num_entries, offset, 2) != 2) {
+            cli_dbgmsg("cli_parsetiff: Failed to acquire number of directory entries in current IFD, file appears to be truncated.\n");
+            cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingNumIFDDirectoryEntries");
+            status = CL_EPARSE;
+            goto done;
+        }
         offset += 2;
         num_entries = tiff16_to_host(big_endian, num_entries);
 
@@ -85,8 +108,12 @@ int cli_parsetiff(cli_ctx *ctx)
 
         /* transverse IFD entries */
         for (i = 0; i < num_entries; i++) {
-            if (fmap_readn(map, &entry, offset, sizeof(entry)) != sizeof(entry))
-                return CL_EPARSE;
+            if (fmap_readn(map, &entry, offset, sizeof(entry)) != sizeof(entry)) {
+                cli_dbgmsg("cli_parsetiff: Failed to read next IFD entry, file appears to be truncated.\n");
+                cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingIFDEntry");
+                status = CL_EPARSE;
+                goto done;
+            }
             offset += sizeof(entry);
 
             entry.tag    = tiff16_to_host(big_endian, entry.tag);
@@ -146,7 +173,8 @@ int cli_parsetiff(cli_ctx *ctx)
                 if (entry.value + value_size > map->len) {
                     cli_warnmsg("cli_parsetiff: TFD entry field %u exceeds bounds of TIFF file [%llu > %llu]\n",
                                 i, (long long unsigned)(entry.value + value_size), (long long unsigned)map->len);
-                    return cli_append_virus(ctx, "Heuristics.TIFF.OutOfBoundsAccess");
+                    status = cli_append_virus(ctx, "Heuristics.Broken.Media.TIFF.OutOfBoundsAccess");
+                    goto done;
                 }
             }
         }
@@ -154,12 +182,24 @@ int cli_parsetiff(cli_ctx *ctx)
         ifd_count++;
 
         /* acquire next IFD location, gets 0 if last IFD */
-        if (fmap_readn(map, &offset, offset, sizeof(offset)) != sizeof(offset))
-            return CL_EPARSE;
+        if (fmap_readn(map, &offset, offset, sizeof(offset)) != sizeof(offset)) {
+            cli_dbgmsg("cli_parsetiff: Failed to aquire next IFD location, file appears to be truncated.\n");
+            cli_append_possibly_unwanted(ctx, "Heuristics.Broken.Media.TIFF.EOFReadingChunkCRC");
+            status = CL_EPARSE;
+            goto done;
+        }
         offset = tiff32_to_host(big_endian, offset);
     } while (offset);
 
     cli_dbgmsg("cli_parsetiff: examined %u IFD(s)\n", ifd_count);
 
-    return CL_SUCCESS;
+    status = CL_CLEAN;
+
+done:
+    if (status == CL_EPARSE) {
+        /* We added with cli_append_possibly_unwanted so it will alert at the end if nothing else matches. */
+        status = CL_CLEAN;
+    }
+
+    return status;
 }
